import { IAppAccessors, IHttp, ILogger, IModify, IPersistence, IRead } from '@rocket.chat/apps-engine/definition/accessors';
import { App } from '@rocket.chat/apps-engine/definition/App';
import { IMessage, IPostMessageSent } from '@rocket.chat/apps-engine/definition/messages';
import { IAppInfo } from '@rocket.chat/apps-engine/definition/metadata';

export class TypeIndicatorApp extends App implements IPostMessageSent {
    private stopTypings = [] as Array<Promise<() => Promise<void>>>;

    constructor(info: IAppInfo, logger: ILogger, accessors: IAppAccessors) {
        super(info, logger, accessors);
    }

    public async executePostMessageSent(message: IMessage, read: IRead, http: IHttp, persistence: IPersistence, modify: IModify): Promise<void> {
        switch (message.text) {
            case 't':
                [
                    undefined, // use app user's name by default
                    'Shiqi Mei', // custom name
                ].forEach((username) => {
                    const stopTyping = modify.getNotifier().typing({ id: message.room.id, username });
                    this.stopTypings.push(stopTyping);
                });
                break;
            default:
                for (const stopTyping of this.stopTypings) {
                    await (await stopTyping)();
                }
                this.stopTypings = [];
        }
    }
}
